// src/entity.ts
import { createId } from "@paralleldrive/cuid2";
import { camelCase, snakeCase } from "change-case";
import { sql } from "kysely";
import { z } from "zod";
var Entity = class {
  constructor(db, name, tables, _zod, id = () => createId(), createdAt = () => Date.now()) {
    this.db = db;
    this.name = name;
    this.id = id;
    this.createdAt = createdAt;
    if (_zod) this.zod = _zod;
    else this.initZodFromTable(tables);
  }
  zod;
  async create(data) {
    const inserted = await this.db.insertInto(this.name).values({
      id: this.id?.(),
      created_at: this.createdAt?.(),
      ...data
      // biome-ignore lint/suspicious/noExplicitAny: <explanation>
    }).returningAll().executeTakeFirstOrThrow();
    return inserted;
  }
  async all(options, pagination) {
    let query = this.db.selectFrom(this.name).selectAll();
    if (options) {
      for (const [key, value] of Object.entries(options)) {
        query = query.where(key, "=", value);
      }
    }
    if (pagination) {
      if (pagination.limit) {
        query = query.limit(pagination.limit);
      }
      if (pagination.offset) {
        query = query.offset(pagination.offset);
      }
    }
    return await query.execute();
  }
  async get(id) {
    return await this.db.selectFrom(this.name).selectAll().where("id", "=", id).executeTakeFirst();
  }
  async update(id, data) {
    const updated = await this.db.updateTable(this.name).set(data).where("id", "=", id).returningAll().executeTakeFirstOrThrow();
    return updated;
  }
  async delete(id) {
    return await this.db.deleteFrom(this.name).where("id", "=", id).execute();
  }
  initZodFromTable(tables) {
    const tableName = snakeCase(this.name);
    const table = tables.find((t) => t.name === tableName);
    if (!table) {
      throw new Error(`Table "${tableName}" not found`);
    }
    const schema = {};
    for (const column of table.columns) {
      const fieldName = camelCase(column.name);
      let zodType;
      switch (column.dataType.toLowerCase()) {
        // String types
        case "varchar":
        case "char":
        case "text":
        case "tinytext":
        case "mediumtext":
        case "longtext":
          zodType = z.string();
          break;
        // Number types
        case "int":
        case "integer":
        case "tinyint":
        case "smallint":
        case "mediumint":
        case "bigint":
        case "int2":
        case "int4":
        case "int8":
          zodType = z.number().int();
          break;
        case "decimal":
        case "numeric":
        case "float":
        case "double":
        case "real":
          zodType = z.number();
          break;
        // Boolean types
        case "boolean":
        case "bool":
          zodType = z.boolean();
          break;
        // Date types
        case "date":
        case "time":
        case "timestamp":
        case "datetime":
          zodType = z.date();
          break;
        // JSON types
        case "json":
        case "jsonb":
          zodType = z.unknown();
          break;
        // UUID type
        case "uuid":
          zodType = z.string().uuid();
          break;
        default:
          zodType = z.unknown();
      }
      if (column.isNullable || fieldName === "createdAt" || fieldName === "id") {
        zodType = zodType.optional();
      }
      schema[fieldName] = zodType;
    }
    this.zod = z.object(schema);
  }
};
async function store(db, opts) {
  const tables = await db.introspection.getTables();
  const entityStore = (key) => {
    const options = opts?.[key];
    return new Entity(
      db,
      key,
      tables,
      options?.zod,
      options?.id,
      options?.createdAt
    );
  };
  entityStore.db = db;
  entityStore.opts = opts;
  return entityStore;
}

// src/database.ts
import { join } from "node:path";
import consola from "consola";
import { Migrator, sql as sql2 } from "kysely";
import {
  DEFAULT_NUMERIC_PARSER,
  DialectManager,
  generate
} from "kysely-codegen";

// src/env.ts
function dev() {
  return process.env.NODE_ENV === "development";
}
function prod() {
  return process.env.NODE_ENV === "production";
}
function test() {
  return process.env.NODE_ENV === "test";
}

// src/database.ts
function migrate(db) {
  return async (...fns) => {
    const migrations = {};
    for (const [idx, fn] of fns.entries()) {
      migrations[idx] = { up: fn };
    }
    const migrator = new Migrator({
      db,
      provider: {
        async getMigrations() {
          return migrations;
        }
      },
      allowUnorderedMigrations: true
    });
    const result = await migrator.migrateToLatest();
    consola.debug(result);
    consola.info("\u2713 migrations applied");
    const dialectManager = new DialectManager({
      numericParser: DEFAULT_NUMERIC_PARSER
    });
    if (!test()) {
      const dialect = dialectManager.getDialect("kysely-bun-sqlite");
      await generate({
        camelCase: true,
        db,
        dialect,
        print: false,
        typeOnlyImports: true,
        runtimeEnums: true,
        outFile: join(
          process.cwd(),
          "node_modules",
          "plainstack",
          "dist",
          "db.d.ts"
        )
      });
    }
    consola.info("\u2713 types generated");
    if (process.env.MIRGATE) {
      console.info("MIGRATE env var set, stopping process");
      process.exit(0);
    }
  };
}
async function rollback(db, fn) {
  const err = null;
  try {
    await sql2.raw("BEGIN").execute(db);
    await fn(db);
  } finally {
    await sql2.raw("ROLLBACK").execute(db);
  }
  if (err) throw err;
}

// src/job.ts
import path from "node:path";
import consola2 from "consola";
import { defineQueue, defineWorker } from "plainjob";
function job(opts) {
  return {
    name: opts.name,
    run: opts.run
  };
}
function schedule(opts) {
  return {
    name: path.parse(opts.name).name,
    cron: opts.cron,
    run: opts.run
  };
}
function queue(opts) {
  const logger = consola2;
  return defineQueue({ connection: opts.connection, logger });
}
async function work(queue2, jobs, schedules) {
  const log = consola2;
  if (!Object.values(jobs).length && !Object.values(schedules).length) {
    log.warn("started worker, but no jobs or schedules to run");
  }
  for (const schedule2 of Object.values(schedules)) {
    log.debug(`scheduling ${schedule2.name} to run at ${schedule2.cron}`);
    queue2.schedule(schedule2.name, { cron: schedule2.cron });
  }
  log.debug(`starting worker for jobs: ${Object.keys(jobs).join(", ")}`);
  if (Object.values(schedules).length) {
    log.debug(
      `starting worker for schedules: ${Object.keys(schedules).join(", ")}`
    );
  }
  const workables = [...Object.values(jobs), ...Object.values(schedules)];
  const workers = workables.map(
    (w) => defineWorker(w.name, w.run, { queue: queue2, logger: log })
  );
  await Promise.all(workers.map((worker) => worker.start()));
}
async function perform(queue2, job2, data) {
  const log = consola2;
  if (test()) {
    log.warn("NODE_ENV=test, performing immediately job in test mode");
    await job2.run({ data });
    return { id: "test" };
  }
  log.debug(`performing job ${job2.name}`);
  return queue2.add(job2.name, { data });
}

// src/form.ts
import { parseWithZod } from "@conform-to/zod";
import { conformValidator } from "@hono/conform-validator";
function form(schema) {
  return conformValidator(
    async (formData) => parseWithZod(formData, { schema: await schema })
  );
}
export {
  dev,
  form,
  job,
  migrate,
  perform,
  prod,
  queue,
  rollback,
  schedule,
  store,
  test,
  work
};
//# sourceMappingURL=plainstack.js.map